@@ -150,12 +150,7 @@ exclPrefixPop(xsltStylesheetPtr style)
  */
 static void
 xsltInit (void) {
-    static int initialized = 0;
-
-    if (initialized == 0) {
-	initialized = 1;
-	xsltRegisterAllExtras();
-    }
+    xsltRegisterAllExtras();
 }
 
 /**
@@ -319,8 +314,10 @@ xsltFreeTemplate(xsltTemplatePtr template) {
     if (template->match) xmlFree(template->match);
     if (template->name) xmlFree(template->name);
     if (template->nameURI) xmlFree(template->nameURI);
+/*
     if (template->mode) xmlFree(template->mode);
     if (template->modeURI) xmlFree(template->modeURI);
+ */
     if (template->inheritedNs) xmlFree(template->inheritedNs);
     memset(template, -1, sizeof(xsltTemplate));
     xmlFree(template);
@@ -372,6 +369,8 @@ xsltNewStylesheet(void) {
     cur->exclPrefixTab = NULL;
     cur->extInfos = NULL;
     cur->extrasNr = 0;
+    cur->internalized = 1;
+    cur->literal_result = 0;
     cur->dict = xmlDictCreate();
 #ifdef WITH_XSLT_DEBUG
     xsltGenericDebug(xsltGenericDebugContext,
@@ -426,7 +425,7 @@ xsltAllocateExtraCtxt(xsltTransformContextPtr ctxt)
 	    for (i = 0;i < ctxt->extrasMax;i++) {
 		ctxt->extras[i].info = NULL;
 		ctxt->extras[i].deallocate = NULL;
-		ctxt->extras[i].val = NULL;
+		ctxt->extras[i].val.ptr = NULL;
 	    }
 
 	} else {
@@ -445,7 +444,7 @@ xsltAllocateExtraCtxt(xsltTransformContextPtr ctxt)
 	    for (i = ctxt->extrasNr;i < ctxt->extrasMax;i++) {
 		ctxt->extras[i].info = NULL;
 		ctxt->extras[i].deallocate = NULL;
-		ctxt->extras[i].val = NULL;
+		ctxt->extras[i].val.ptr = NULL;
 	    }
 	}
     }
@@ -1178,6 +1177,13 @@ xsltParseStylesheetExcludePrefix(xsltStylesheetPtr style, xmlNodePtr cur) {
 static void
 xsltPrecomputeStylesheet(xsltStylesheetPtr style, xmlNodePtr cur) {
     xmlNodePtr delete;
+    int internalize = 0;
+
+    if ((style != NULL) && (cur != NULL) && (cur->doc != NULL) &&
+        (style->dict != NULL) && (cur->doc->dict == style->dict))
+	internalize = 1;
+    else
+        style->internalized = 0;
 
     /*
      * This content comes from the stylesheet
@@ -1198,6 +1204,34 @@ xsltPrecomputeStylesheet(xsltStylesheetPtr style, xmlNodePtr cur) {
 	if (cur->type == XML_ELEMENT_NODE) {
 	    int exclPrefixes;
 
+	    /*
+	     * Internalize attributes values.
+	     */
+	    if ((internalize) && (cur->properties != NULL)) {
+	        xmlAttrPtr prop = cur->properties;
+		xmlNodePtr txt;
+
+		while (prop != NULL) {
+		    txt = prop->children;
+		    if ((txt != NULL) && (txt->type == XML_TEXT_NODE) &&
+		        (txt->content != NULL) &&
+			(!xmlDictOwns(style->dict, txt->content))) {
+			xmlChar *old = (xmlChar *) txt->content;
+
+			/*
+			 * internalize the text string, goal is to speed
+			 * up operations and minimize used space by compiled
+			 * stylesheets.
+			 */
+			txt->content = (xmlChar *)
+			               xmlDictLookup(style->dict, old, -1);
+			if (old != txt->content)
+			    xmlFree(old);
+		    }
+		    prop = prop->next;
+		}
+	    }
+
 	    exclPrefixes = xsltParseStylesheetExcludePrefix(style, cur);
 	    if (IS_XSLT_ELEM(cur)) {
 		xsltStylePreCompute(style, cur);
@@ -1264,6 +1298,17 @@ xsltPrecomputeStylesheet(xsltStylesheetPtr style, xmlNodePtr cur) {
 		if (xmlNodeGetSpacePreserve(cur) != 1) {
 		    delete = cur;
 		}
+	    } else if ((cur->content != NULL) && (internalize) &&
+	               (!xmlDictOwns(style->dict, cur->content))) {
+		xmlChar *old = (xmlChar *) cur->content;
+
+		/*
+		 * internalize the text string, goal is to speed
+		 * up operations and minimize used space by compiled
+		 * stylesheets.
+		 */
+		cur->content = (xmlChar *) xmlDictLookup(style->dict, old, -1);
+		xmlFree(old);
 	    }
 	} else if ((cur->type != XML_ELEMENT_NODE) &&
 		   (cur->type != XML_CDATA_SECTION_NODE)) {
@@ -1473,8 +1518,31 @@ xsltParseTemplateContent(xsltStylesheetPtr style, xmlNodePtr templ) {
 		     * replace xsl:text by the list of childs
 		     */
 		    if (text == NULL) {
+		        int internalize = 0;
+			if ((style != NULL) && (text != NULL) &&
+			    (text->doc != NULL) && (style->dict != NULL) &&
+			    (text->doc->dict == style->dict))
+			    internalize = 1;
+			else
+			    style->internalized = 0;
+
 			text = cur->children;
 			while (text != NULL) {
+			    if ((text->content != NULL) &&
+			        (!xmlDictOwns(style->dict, text->content))) {
+
+				/*
+				 * internalize the text string
+				 */
+				if (text->doc->dict != NULL) {
+				    xmlChar *old = (xmlChar *) text->content;
+				    text->content = 
+					(xmlChar *) xmlDictLookup(
+						text->doc->dict, old, -1);
+				    xmlFree(old);
+				}
+			    }
+
 			    next = text->next;
 			    xmlUnlinkNode(text);
 			    xmlAddPrevSibling(cur, text);
@@ -1713,15 +1781,14 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
 	    if (URI != NULL)
 		modeURI = xmlStrdup(URI);
 	}
-	ret->mode = mode;
-	ret->modeURI = modeURI;
+	ret->mode = xmlDictLookup(style->dict, mode, -1);
+	ret->modeURI = xmlDictLookup(style->dict, modeURI, -1);
 #ifdef WITH_XSLT_DEBUG_PARSING
 	xsltGenericDebug(xsltGenericDebugContext,
 	     "xsltParseStylesheetTemplate: mode %s\n", mode);
 #endif
-    } else {
-	mode = NULL;
-	modeURI = NULL;
+        if (mode != NULL) xmlFree(mode);
+	if (modeURI != NULL) xmlFree(modeURI);
     }
     prop = xsltGetNsProp(template, (const xmlChar *)"match", XSLT_NAMESPACE);
     if (prop != NULL) {
@@ -1739,11 +1806,16 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
     prop = xsltGetNsProp(template, (const xmlChar *)"name", XSLT_NAMESPACE);
     if (prop != NULL) {
         const xmlChar *URI;
+	xsltTemplatePtr cur;
 
-	if (ret->name != NULL) xmlFree(ret->name);
-	ret->name = NULL;
-	if (ret->nameURI != NULL) xmlFree(ret->nameURI);
-	ret->nameURI = NULL;
+	if (ret->name != NULL) {
+	    xmlFree(ret->name);
+	    ret->name = NULL;
+	}
+	if (ret->nameURI != NULL) {
+	    xmlFree(ret->nameURI);
+	    ret->nameURI = NULL;
+	}
 
 	URI = xsltGetQNameURI(template, &prop);
 	if (prop == NULL) {
@@ -1761,6 +1833,19 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
 		ret->nameURI = xmlStrdup(URI);
 	    else
 		ret->nameURI = NULL;
+	    cur = ret->next;
+	    while (cur != NULL) {
+	        if ((URI != NULL && xmlStrEqual(cur->name, prop) &&
+				xmlStrEqual(cur->nameURI, URI) ) ||
+		    (URI == NULL && cur->nameURI == NULL &&
+				xmlStrEqual(cur->name, prop))) {
+		    xsltTransformError(NULL, style, template,
+		        "xsl:template: error duplicate name '%s'\n", prop);
+		    style->errors++;
+		    goto error;
+		}
+		cur = cur->next;
+	    }
 	}
     }
 
@@ -1770,7 +1855,7 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
     xsltParseTemplateContent(style, template);
     ret->elem = template;
     ret->content = template->children;
-    xsltAddTemplate(style, ret, mode, modeURI);
+    xsltAddTemplate(style, ret, ret->mode, ret->modeURI);
 
 error:
     for (;exclPrefixes > 0;exclPrefixes--)
@@ -1812,8 +1897,6 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
 	xmlFree(prop);
     }
 
-    xsltParseStylesheetExtPrefix(style, top);
-
     cur = top->children;
 
     /*
@@ -1959,8 +2042,6 @@ xsltParseStylesheetProcess(xsltStylesheetPtr ret, xmlDocPtr doc) {
 	return(NULL);
     }
     xsltParseStylesheetExcludePrefix(ret, cur);
-    xsltPrecomputeStylesheet(ret, cur);
-
     if ((IS_XSLT_ELEM(cur)) && 
 	((IS_XSLT_NAME(cur, "stylesheet")) ||
 	 (IS_XSLT_NAME(cur, "transform")))) {
@@ -1968,6 +2049,16 @@ xsltParseStylesheetProcess(xsltStylesheetPtr ret, xmlDocPtr doc) {
 	xsltGenericDebug(xsltGenericDebugContext,
 		"xsltParseStylesheetProcess : found stylesheet\n");
 #endif
+	ret->literal_result = 0;
+
+	xsltParseStylesheetExtPrefix(ret, cur);
+    } else {
+	ret->literal_result = 1;
+    }
+    if (!ret->nopreproc)
+	xsltPrecomputeStylesheet(ret, cur);
+
+    if (ret->literal_result == 0) {
 
 	xsltParseStylesheetTop(ret, cur);
     } else {
@@ -2015,6 +2106,7 @@ xsltParseStylesheetProcess(xsltStylesheetPtr ret, xmlDocPtr doc) {
 	template->elem = (xmlNodePtr) doc;
 	template->content = doc->children;
 	xsltAddTemplate(ret, template, NULL, NULL);
+	ret->literal_result = 1;
     }
 
     return(ret);
